from datetime import datetime
from ergondata_executions.interfaces import *
from ergondata_executions.v1.queue.interfaces import Queue, QueueItemStatus
from pydantic import StrictFloat, validator
from typing import Annotated


class QueueHighDurationEvent(BaseModel):
    duration_metric: Literal["absolute", "mean", "median"]
    duration_value: Union[StrictInt, StrictFloat]

    @validator('duration_value', pre=True, always=True)
    def check_duration_value(cls, duration_value, values):
        duration_metric = values.get('duration_metric')
        if duration_metric == "absolute":
            isinstance(duration_value, StrictInt)
        else:
            if not (duration_value, StrictFloat):
                raise ValidationError
            if duration_value > 1:
                raise ValidationError
            if duration_value < 0:
                raise ValidationError
        return duration_value

class QueueEventSubscription(BaseModel):
    id: StrictStr
    event: Any
    whatsapp_integration: StrictBool
    whatsapp_recipients: Optional[Union[List[WhatsappRecipient], None]] = None
    email_integration: StrictBool
    email_recipients: Optional[Union[List[StrictStr], None]] = None
    webhook_integration: StrictBool
    webhook_url: Optional[Union[StrictStr, None]] = None
    queue_exception_id: Optional[StrictStr] = None
    queue: Queue
    payload: Union[Any, QueueHighDurationEvent] = None
    created_at: datetime
    updated_at: datetime

class CreateQueueEventSubscriptionRequestPayload(BaseModel):
    event_id: StrictStr
    whatsapp_integration: StrictBool = False
    whatsapp_recipients: Optional[Union[List[WhatsappRecipient], None]] = None
    email_integration: StrictBool = False
    email_recipients: Optional[Union[List[StrictStr], None]] = None
    webhook_integration: StrictBool = False
    webhook_url: Optional[Union[StrictStr, None]] = None
    queue_exception_id: Optional[StrictStr] = None
    payload: Union[Any, QueueHighDurationEvent] = None


class UpdateQueueEventSubscriptionRequestPayload(CreateQueueEventSubscriptionRequestPayload):
    pass

class CreateQueueEventSubscriptionResponsePayload(APIBaseResponse):
    data: Optional[QueueEventSubscription] = None


class DeleteQueueEventSubscriptionResponsePayload(APIBaseResponse):
    queue_event_subscription_id: Optional[StrictStr] = None


class GetQueueEventSubscriptionsResponsePayload(APIBaseResponse):
    data: List[QueueEventSubscription]


class UpdateQueueEventSubscriptionResponsePayload(CreateQueueEventSubscriptionResponsePayload):
    data: Optional[QueueEventSubscription] = None




import requests

from ergondata_executions.v1.auth.controller import AuthController
from ergondata_executions.v1.queue_priority.interfaces import *
from ergondata_executions.v1.decorators import *

class QueuePriorityController:

    CREATE_QUEUE_PRIORITY_URL = "processes/{0}/queues/{1}/priorities"
    GET_QUEUE_PRIORITIES_URL = "processes/{0}/queues/{1}/priorities"
    DELETE_QUEUE_PRIORITY_URL = "processes/{0}/queues/{1}/priorities/{2}"
    UPDATE_QUEUE_PRIORITY_URL = "processes/{0}/queues/{1}/priorities/{2}"

    def __init__(self, api_client: AuthController):
        self.api_client = api_client

    @api_request(log_message="Creating queue_priority {name}", out_schema=CreateQueuePriorityResponsePayload)
    def create_queue_priority(
        self,
        process_id: StrictStr,
        queue_id: StrictStr,
        name: StrictStr,
        description: StrictStr,
    ) -> CreateQueuePriorityResponsePayload:
        res: Any = requests.post(
            url=f"{self.api_client.ROOT_URL}{self.CREATE_QUEUE_PRIORITY_URL.format(process_id, queue_id)}",
            json=CreateQueuePriorityRequestPayload(
                name=name,
                description=description
            ).model_dump(),
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res

    @api_request(
        log_message="Deleting queue_priority {queue_priority_id}",
        out_schema=DeleteQueuePriorityResponsePayload
    )
    def delete_queue_priority(
        self,
        process_id: StrictStr,
        queue_id: StrictStr,
        queue_priority_id: StrictStr
    ) -> DeleteQueuePriorityResponsePayload:
        res: Any = requests.delete(
            url=f"{self.api_client.ROOT_URL}{self.DELETE_QUEUE_PRIORITY_URL.format(process_id, queue_id, queue_priority_id)}",
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res

    @api_request(log_message="Getting queue_priorities", out_schema=GetQueuePrioritiesResponsePayload)
    def get_queue_priorities(self, process_id: StrictStr, queue_id: StrictStr) -> GetQueuePrioritiesResponsePayload:
        res: Any = requests.get(
            url=f"{self.api_client.ROOT_URL}{self.GET_QUEUE_PRIORITIES_URL.format(process_id, queue_id)}",
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res

    @api_request(log_message="Updating queue_priority {name}", out_schema=UpdateQueuePriorityResponsePayload)
    def update_queue_priority(
        self,
        process_id: StrictStr,
        queue_priority_id: StrictStr,
        queue_id: StrictStr,
        name: StrictStr,
        description: StrictStr,
    ) -> UpdateQueuePriorityResponsePayload:
        res: Any = requests.put(
            url=f"{self.api_client.ROOT_URL}{self.UPDATE_QUEUE_PRIORITY_URL.format(process_id, queue_id, queue_priority_id)}",
            json=UpdateQueuePriorityRequestPayload(
                name=name,
                description=description,
            ).model_dump(),
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res



from ergondata_executions.interfaces import *
from ergondata_executions.v1.queue.interfaces import Queue, QueueItemStatus
from ergondata_executions.v1.task_execution.interfaces import TaskExecution
from ergondata_executions.v1.queue_exception.interfaces import QueueException
from ergondata_executions.v1.queue_priority.interfaces import QueuePriority
from typing import Any
from ergondata_executions.interfaces import APIPagination

class QueueItem(BaseModel):
    id: StrictStr
    processing_status: QueueItemStatus
    payload: Any
    queue: Queue
    retries_within_count: StrictInt
    retries_outside_count: StrictInt
    created_at: datetime
    processing_status_message: Optional[StrictStr] = None
    processing_exception: Optional[QueueException] = None
    processing_priority: Optional[QueuePriority] = None
    external_id: Optional[StrictStr] = None
    task_producer_execution: Optional[TaskExecution] = None
    task_consumer_execution: Optional[TaskExecution] = None
    tags: Optional[List[StrictStr]] = None
    started_at: Optional[datetime] = None
    finished_at: Optional[datetime] = None


class CreateQueueItemRequestPayload(BaseModel):
    payload: Any
    external_id: Optional[StrictStr] = None
    processing_status_id: Optional[Literal["pending", "success", "system_error", "business_exception"]] = "pending"
    processing_status_message: Optional[StrictStr] = None
    processing_exception_id: Optional[StrictStr] = None
    processing_priority_id: Optional[StrictStr] = None
    tags: Optional[List[StrictStr]] = None


class CreateQueueItemResponsePayload(APIBaseResponse):
    data: Optional[Union[QueueItem, None]] = None


class GetQueueItemQueryParams(BaseModel):
    priority_id__in: Optional[list[StrictStr]] = None
    external_id: Optional[StrictStr] = None


class GetQueueItemResponsePayload(APIBaseResponse):
    data: Optional[QueueItem] = None


class UpdateQueueItemRequestPayload(BaseModel):
    payload: Optional[Any] = None
    processing_status_id: Optional[Literal["success", "system_error", "business_exception"]] = None
    processing_status_message: Optional[StrictStr] = None
    processing_exception_id: Optional[StrictStr] = None


class UpdateQueueItemResponsePayload(CreateQueueItemResponsePayload):
    pass


class GetQueueItemsQueryParams(BaseModel):
    page: StrictStr = 1
    page_size: StrictStr = 10
    id: Optional[StrictStr] = None
    id__in: Optional[List[StrictStr]] = None
    external_id: Optional[StrictStr] = None
    external_id__in: Optional[List[StrictStr]] = None
    external_id__contains: Optional[StrictStr] = None
    tags__contains: Optional[StrictStr] = None
    tags__exact: Optional[List[StrictStr]] = None
    processing_priority_id: Optional[StrictStr] = None
    processing_priority_id__in: Optional[list[StrictStr]] = None
    processing_status_id: Optional[StrictStr] = None
    processing_status_id__in: Optional[List[StrictStr]] = None
    task_producer_execution_id: Optional[StrictStr] = None
    task_producer_execution_id__in: Optional[List[StrictStr]] = None
    task_consumer_execution_id: Optional[StrictStr] = None
    task_consumer_execution_id__in: Optional[List[StrictStr]] = None
    task_producer_execution__worker_id: Optional[StrictStr] = None
    task_producer_execution__worker_id__in: Optional[List[StrictStr]] = None
    task_consumer_execution__worker_id: Optional[StrictStr] = None
    task_consumer_execution__worker_id__in: Optional[List[StrictStr]] = None
    produced_by_user_id: Optional[StrictStr] = None
    produced_by_user_id__in: Optional[List[StrictStr]] = None
    consumed_by_user_id: Optional[StrictStr] = None
    consumed_by_user_id__in: Optional[List[StrictStr]] = None
    processing_exception_id: Optional[StrictStr] = None
    processing_exception_id__in: Optional[List[StrictStr]] = None
    processing_status_message__contains: Optional[StrictStr] = None
    max_retries_within_execution: Optional[StrictInt] = None
    max_retries_within_execution__lt: Optional[StrictInt] = None
    max_retries_within_execution__lte: Optional[StrictInt] = None
    max_retries_within_execution__gt: Optional[StrictInt] = None
    max_retries_within_execution__gte: Optional[StrictInt] = None
    max_retries_outside_execution: Optional[StrictInt] = None
    max_retries_outside_execution__gt: Optional[StrictInt] = None
    max_retries_outside_execution__gte: Optional[StrictInt] = None
    max_retries_outside_execution__lt: Optional[StrictInt] = None
    max_retries_outside_execution__lte: Optional[StrictInt] = None
    created_at__lte: Optional[datetime] = None
    created_at__lt: Optional[datetime] = None
    created_at__gt: Optional[datetime] = None
    created_at__gte: Optional[datetime] = None
    started_at__lte: Optional[datetime] = None
    started_at__lt: Optional[datetime] = None
    started_at__gt: Optional[datetime] = None
    started_at__gte: Optional[datetime] = None
    finished_at__lte: Optional[datetime] = None
    finished_at__lt: Optional[datetime] = None
    finished_at__gt: Optional[datetime] = None
    finished_at__gte: Optional[datetime] = None
    payload: Optional[StrictStr] = None
    payload__contains: Optional[StrictStr] = None

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def to_dict(self):

        original_dict = self.dict()
        new_dict = original_dict.copy()
        public_keys = original_dict.items()

        for key, value in public_keys:
            if isinstance(value, list):
                new_dict[key] = ','.join(value)

        return new_dict

class GetQueueItemsResponsePayload(APIBaseResponse):
    data: Optional[List[QueueItem]] = None
    pagination: Optional[APIPagination] = None


class GetQueueItemsReporter(BaseModel):
    queue_id: StrictStr
    data: List[QueueItem]


import requests

from ergondata_executions.v1.auth.controller import AuthController
from ergondata_executions.v1.task_execution_log.interfaces import *
from ergondata_executions.v1.decorators import *
from typing import Literal

class TaskExecutionLogController:

    CREATE_TASK_EXECUTION_LOG_URL = "processes/{0}/tasks/{1}/executions/{2}/logs"
    GET_TASK_EXECUTION_LOGS_URL = "processes/{0}/tasks/{1}/executions/{2}/logs"

    def __init__(self, api_client: AuthController):
        self.api_client = api_client

    @api_request(out_schema=CreateTaskExecutionLogResponse, log_response=False)
    def write_tk_exec_log(
        self,
        message: StrictStr,
        level: Literal["info", "warning", "error"] = "info",
        task_step_id: Optional[StrictStr] = None,
        write_to_server: Optional[StrictBool] = True
    ) -> CreateTaskExecutionLogResponse | None:

        self.api_client.log(
            message=message,
            level=level
        )

        if not write_to_server:
            return

        res: Any = requests.post(
            url=f"{self.api_client.ROOT_URL}{self.CREATE_TASK_EXECUTION_LOG_URL.format(self.api_client.process_id, self.api_client.task_id, self.api_client.task_execution_id)}",
            json=CreateTaskExecutionLogRequest(
                message=message,
                level_id=level,
                task_step_id=task_step_id
            ).model_dump(),
            headers=self.api_client.exec_header,
            timeout=self.api_client.timeout
        )
        return res

    @api_request(log_message="Getting task execution logs", out_schema=GetTaskExecutionLogsResponse)
    def get_task_execution_logs(
        self,
        process_id: StrictStr,
        task_id: StrictStr,
        task_execution_id: StrictStr,
        page: StrictInt = 1,
        page_size: StrictInt = 100,
        message__contains: Optional[StrictStr] = None,
        message__starts_with: Optional[StrictStr] = None,
        message__regex: Optional[StrictStr] = None,
        level_id: Optional[StrictStr] = None,
        level_id__in: Optional[List[StrictStr]] = None,
        task_step_id__contains: Optional[StrictStr] = None,
        task_step_id__starts_with: Optional[StrictStr] = None,
        task_step_id__regex: Optional[StrictStr] = None,
        created_at_lte: Optional[datetime] = None,
        created_at_lt: Optional[datetime] = None,
        created_at_gt: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
    ) -> GetTaskExecutionLogsResponse:
        res: Any = requests.get(
            url=f"{self.api_client.ROOT_URL}{self.GET_TASK_EXECUTION_LOGS_URL.format(process_id, task_id, task_execution_id)}",
            params=GetTaskExecutionLogsQueryParams(
                task_id=task_id,
                task_execution_id=task_execution_id,
                page=page,
                page_size=page_size,
                message__contains=message__contains,
                message__regex=message__regex,
                message__starts_with=message__starts_with,
                level_id=level_id,
                level_id__in=','.join(level_id__in),
                task_step_id__contains=task_step_id__contains,
                task_step_id__regex=task_step_id__regex,
                task_step_id__starts_with=task_step_id__starts_with,
                created_at_lte=created_at_lte,
                created_at_lt=created_at_lt,
                created_at_gt=created_at_gt,
                created_at_gte=created_at_gte

            ).model_dump(),
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res


import requests
from ergondata_executions.v1.auth.interfaces import *
from ergondata_executions.v1.api_logger import APILogger
from ergondata_executions.v1.queue_item.interfaces import QueueItem


class AuthController(APILogger):

    ROOT_URL = "https://executions.ergondata.com.br/api/v1/"
    # ROOT_URL = "http://127.0.0.1:8000/api/v1/"
    AUTH_URL = "auth"

    auth_token: StrictStr = None
    process_id: StrictStr = None
    task_id: StrictStr = None
    task_execution_id: StrictStr = None
    task_execution_token: StrictStr = None
    queue_item: QueueItem

    def __init__(
        self,
        auth: AuthRequestPayload,
        config: APIConfig = APIConfig()
    ):

        super(AuthController, self).__init__(
            enable_logs=config.enable_logs,
            log_file_path=config.log_file_path,
            preserve_logger_handler=config.preserve_logger_handler
        )

        self.auth = auth
        self.timeout = config.api_timeout
        self.auth_token = self.__authenticate().token
        self.exec_token = None
        self.config = config

    def set_task_execution(
        self,
        exec_token: StrictStr,
        process_id: StrictStr,
        task_id: StrictStr,
        task_execution_id: StrictStr
    ):
        self.exec_token = exec_token
        self.process_id = process_id
        self.task_id = task_id
        self.task_execution_id = task_execution_id


    @property
    def auth_header(self):
        return {"Authorization": f"Bearer {self.auth_token}"}

    @property
    def exec_header(self):
        return {"Authorization": f"Bearer {self.exec_token}"}

    def __authenticate(self) -> AuthResponsePayload:

        try:

            self.logger.info(f"Authenticating user { self.auth.username }")

            res = requests.post(
                url=f"{self.ROOT_URL}{self.AUTH_URL}",
                json=self.auth.model_dump(),
                timeout=self.timeout
            )

            if res.status_code == 200:
                clean_response = res.json()
                del clean_response['token']
                clean_response = AuthResponsePayload(**clean_response)
                self.logger.info(clean_response)
            else:
                self.logger.error(AuthResponsePayload(**res.json()))

            return AuthResponsePayload(**res.json())

        except BaseException as e:

            self.logger.info(f"Failed to authenticate user. {e}")
            response = AuthResponsePayload(
                status="error",
                message=str(e)
            )
            return response





import logging
import os
import uuid
from typing import Literal
from pydantic import StrictBool


class APILogger:

    _file_handlers = []

    def __init__(
        self,
        enable_logs: bool = True,
        log_file_path: str = 'logs',
        log_level: Literal["info", "debug", "error", "warning"] = "debug",
        preserve_logger_handler: StrictBool = True
    ):
        self.enable_logs = enable_logs
        self.log_file_path = log_file_path
        self.logger = self._create_logger()
        self.log_level = log_level
        self.preserve_logger_handler = preserve_logger_handler

        # Configure the logger only if it hasn't been configured yet
        self._configure_logger()

    def _create_logger(self):
        logger = logging.getLogger(str(uuid.uuid4()))  # Use a unique name for each logger instance
        return logger

    def _configure_logger(self):
        # Clear existing handlers if preserve_logger_handler is set to False

        self._configure_file_handler()
        console_handler = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        console_handler.setFormatter(formatter)
        self.logger.addHandler(console_handler)

        # Set the logger level to DEBUG to capture all messages
        if self.log_level == "debug":
            self.logger.setLevel(logging.DEBUG)
        elif self.log_level == "info":
            self.logger.setLevel(logging.INFO)
        elif self.log_level == "warning":
            self.logger.setLevel(logging.WARNING)
        elif self.log_level == "error":
            self.logger.setLevel(logging.ERROR)
        else:
            self.logger.setLevel(logging.DEBUG)

    def _configure_file_handler(self):

        if not self.log_file_path:
            return

        log_file_name = self._get_log_file_name()
        log_file_path = os.path.join(self.log_file_path, log_file_name)

        if self.preserve_logger_handler and APILogger._file_handlers:
            self.logger.addHandler(APILogger._file_handlers[0])
        else:
            file_handler = logging.FileHandler(log_file_path)
            formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
            file_handler.setFormatter(formatter)
            self.logger.addHandler(file_handler)
            APILogger._file_handlers = [file_handler]

    def _get_log_file_name(self):
        # Generate a unique filename using UUID
        return f"{uuid.uuid4()}.txt"

    def log(self, message, level: Literal["warning", "info", "error"] = "info"):
        if level == "warning":
            self.logger.log(msg=message, level=logging.WARNING)
        elif level == "info":
            self.logger.log(msg=message, level=logging.INFO)
        elif level == "error":
            self.logger.log(msg=message, level=logging.ERROR)
        else:
            pass



import sys

import wrapt

from ergondata_executions.v1.auth.controller import *
from ergondata_executions.v1.auth.interfaces import AuthRequestPayload
from ergondata_executions.v1.workspace.controller import WorkspaceController
from ergondata_executions.v1.process.controller import ProcessController
from ergondata_executions.v1.task.controller import TaskController
from ergondata_executions.v1.task_exception.controller import TaskExceptionController
from ergondata_executions.v1.task_event_subscription.controller import TaskEventSubscriptionController
from ergondata_executions.v1.worker.controller import WorkerController
from ergondata_executions.v1.task_execution.controller import TaskExecutionController
from ergondata_executions.v1.task_execution_log.controller import TaskExecutionLogController
from ergondata_executions.v1.queue.controller import QueueController
from ergondata_executions.v1.queue_exception.controller import QueueExceptionController
from ergondata_executions.v1.queue_event_subscription.controller import QueueEventSubscriptionController
from ergondata_executions.v1.queue_priority.controller import QueuePriorityController
from ergondata_executions.v1.queue_item.controller import QueueItemController



class ErgonClient(
    WorkspaceController,
    ProcessController,
    TaskController,
    TaskExceptionController,
    TaskEventSubscriptionController,
    WorkerController,
    TaskExecutionController,
    TaskExecutionLogController,
    QueueController,
    QueueExceptionController,
    QueueEventSubscriptionController,
    QueuePriorityController,
    QueueItemController
):

    def __init__(
        self,
        auth: AuthRequestPayload,
        config: APIConfig = APIConfig()
    ):

        self.api_client = AuthController(
            auth=auth,
            config=config
        )

        super().__init__(api_client=self.api_client)

def task_execution(auth: AuthRequestPayload, config: APIConfig = APIConfig()):

    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        ergon = ErgonClient(auth=auth, config=config)
        execution = ergon.create_task_execution(
            process_id=ergon.api_client.config.task_exec_config.process_id,
            task_id=ergon.api_client.config.task_exec_config.task_id
        )

        if execution.status != "success":
            ergon.write_tk_exec_log(message="Failed to create task execution", write_to_server=False)
            sys.exit()

        ergon.write_tk_exec_log(message="Task execution started")
        result = wrapped(ergon, *args, **kwargs)
        return result

    return wrapper



from ergondata_executions.interfaces import *
from ergondata_executions.v1.task.interfaces import Task
from pydantic import StrictFloat


class TaskHighDurationEvent(BaseModel):
    duration_metric: Literal["absolute", "mean", "median"]
    duration_value: Union[StrictInt, StrictFloat]


class TaskDelayedEvent(BaseModel):
    delay_tolerance: Union[StrictInt, StrictFloat]


class TaskEventSubscription(BaseModel):
    id: StrictStr
    event: Any
    whatsapp_integration: StrictBool
    whatsapp_recipients: Optional[Union[List[WhatsappRecipient], None]] = None
    email_integration: StrictBool
    email_recipients: Optional[Union[List[StrictStr], None]] = None
    webhook_integration: StrictBool
    webhook_url: Optional[Union[StrictStr, None]] = None
    task_exception_id: Optional[StrictStr] = None
    task: Task
    payload: Union[TaskHighDurationEvent, TaskDelayedEvent, None] = None
    created_at: datetime
    updated_at: datetime


class CreateTaskEventSubscriptionRequestPayload(BaseModel):
    event_id: StrictStr
    whatsapp_integration: StrictBool = False
    whatsapp_recipients: Optional[Union[List[WhatsappRecipient], None]] = None
    email_integration: StrictBool = False
    email_recipients: Optional[Union[List[StrictStr], None]] = None
    webhook_integration: StrictBool = False
    webhook_url: Optional[Union[StrictStr, None]] = None
    task_exception_id: Optional[StrictStr] = None
    payload: Union[TaskHighDurationEvent, TaskDelayedEvent, None] = None


class UpdateTaskEventSubscriptionRequestPayload(CreateTaskEventSubscriptionRequestPayload):
    pass


class CreateTaskEventSubscriptionResponsePayload(APIBaseResponse):
    data: Optional[TaskEventSubscription] = None


class DeleteTaskEventSubscriptionResponsePayload(APIBaseResponse):
    process_id: Optional[StrictStr] = None


class GetTaskEventSubscriptionsResponsePayload(APIBaseResponse):
    data: List[TaskEventSubscription]


class UpdateTaskEventSubscriptionResponsePayload(CreateTaskEventSubscriptionResponsePayload):
    data: Optional[TaskEventSubscription] = None




import requests

from ergondata_executions.v1.auth.controller import AuthController
from ergondata_executions.v1.task_execution.interfaces import *
from ergondata_executions.v1.decorators import *

class TaskExecutionController:

    CREATE_TASK_EXECUTION_URL = "processes/{0}/tasks/{1}/executions"
    GET_TASK_EXECUTIONS_URL = "processes/{0}/tasks/{1}/executions"
    UPDATE_TASK_EXECUTION_URL = "processes/{0}/tasks/{1}/executions/{2}"

    def __init__(self, api_client: AuthController):
        self.api_client = api_client

    @api_request(log_message="Creating task execution", out_schema=CreateTaskExecutionResponsePayload, exec_token=True)
    def create_task_execution(
        self,
        process_id: StrictStr,
        task_id: StrictStr,
        dev_mode: StrictBool = True,
        processing_status_message: StrictBool = None
    ) -> CreateTaskExecutionResponsePayload:
        res: Any = requests.post(
            url=f"{self.api_client.ROOT_URL}{self.CREATE_TASK_EXECUTION_URL.format(process_id, task_id)}",
            json=CreateTaskExecutionRequestPayload(
                dev_mode=dev_mode,
                processing_status_message=processing_status_message
            ).model_dump(),
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res

    @api_request(log_message="Updating task execution  status", out_schema=UpdateTaskExecutionResponsePayload)
    def update_task_execution(
        self,
        process_id: StrictStr = None,
        task_execution_id: StrictStr = None,
        task_id: StrictStr = None,
        processing_status_id: Literal["success", "system_error", "business_exception", None] = None,
        task_exception_id: StrictStr = None,
        processing_status_message: StrictStr = None
    ) -> UpdateTaskExecutionResponsePayload:
        if task_execution_id:
            url = self.UPDATE_TASK_EXECUTION_URL.format(process_id, task_id, task_execution_id)
        else:
            url = self.UPDATE_TASK_EXECUTION_URL.format(self.api_client.process_id, self.api_client.task_id, self.api_client.task_execution_id)
        res: Any = requests.put(
            url=f"{self.api_client.ROOT_URL}{url}",
            json=UpdateTaskExecutionRequestPayload(
                task_exception_id=task_exception_id,
                processing_status_message=processing_status_message,
                processing_status_id=processing_status_id
            ).model_dump(),
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res

    @api_request(log_message="Getting task executions", out_schema=GetTaskExecutionsResponsePayload)
    def get_task_executions(
        self,
        process_id: StrictStr,
        task_id: StrictStr,
        page: StrictInt = 1,
        page_size: StrictInt = 100,
        id__in: Optional[List[StrictStr]] = None,
        processing_status_id__exact: Optional[Literal["success", "system_error", "business_exception", "processing"]] = None,
        processing_status_id__in: Optional[List[Literal["success", "system_error", "business_exception", "processing", "reset", "released"]]] = None,
        processing_exception_id__exact: Optional[StrictStr] = None,
        processing_exception_id__in: Optional[List[StrictStr]] = None,
        worker_id: Optional[StrictStr] = None,
        worker_id__in: Optional[StrictStr] = None,
        created_at_lte: Optional[datetime] = None,
        created_at_lt: Optional[datetime] = None,
        created_at_gt: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        finished_at_lte: Optional[datetime] = None,
        finished_at_lt: Optional[datetime] = None,
        finished_at_gt: Optional[datetime] = None,
        finished_at_gte: Optional[datetime] = None
    ) -> GetTaskExecutionsResponsePayload:
        res: Any = requests.get(
            url=f"{self.api_client.ROOT_URL}{self.GET_TASK_EXECUTIONS_URL.format(process_id, task_id)}",
            params=GetTaskExecutionsQueryParams(
                page=page,
                page_size=page_size,
                id__in=id__in,
                processing_status_id__exact=processing_status_id__exact,
                processing_status_id__in=processing_status_id__in,
                processing_exception_id__exact=processing_exception_id__exact,
                processing_exception_id__in=processing_exception_id__in,
                worker_id=worker_id,
                worker_id__in=worker_id__in,
                created_at_lte=created_at_lte,
                created_at_lt=created_at_lt,
                created_at_gt=created_at_gt,
                created_at_gte=created_at_gte,
                finished_at_lte=finished_at_lte,
                finished_at_lt=finished_at_lt,
                finished_at_gt=finished_at_gt,
                finished_at_gte=finished_at_gte

            ).model_dump(),
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res


from ergondata_executions.interfaces import *

class Workspace(BaseModel):
    id: StrictStr
    name: StrictStr
    description: StrictStr
    created_at: datetime
    updated_at: datetime


class CreateWkRequestPayload(BaseModel):
    name: StrictStr
    description: StrictStr

class UpdateWkRequestPayload(CreateWkRequestPayload):
    pass

class CreateWKResponsePayload(APIBaseResponse):
    data: Optional[Workspace] = None

class DeleteWKResponsePayload(APIBaseResponse):
    workspace_id: Optional[StrictStr] = None


class GetWorkspacesResponsePayload(APIBaseResponse):
    data: List[Workspace]

class UpdateWorkspaceResponsePayload(CreateWKResponsePayload):
    pass


import requests

from ergondata_executions.v1.auth.controller import AuthController
from ergondata_executions.v1.task_event_subscription.interfaces import *
from ergondata_executions.v1.decorators import *

class TaskEventSubscriptionController:

    CREATE_TASK_EVENT_SUBSCRIPTION_URL = "processes/{0}/tasks/{1}/event-subscriptions"
    GET_TASK_EVENT_SUBSCRIPTIONS_URL = "processes/{0}/tasks/{1}/event-subscriptions"
    DELETE_TASK_EVENT_SUBSCRIPTION_URL = "processes/{0}/tasks/{1}/event-subscriptions/{2}"
    UPDATE_TASK_EVENT_SUBSCRIPTION_URL = "processes/{0}/tasks/{1}/event-subscriptions/{2}"

    def __init__(self, api_client: AuthController):
        self.api_client = api_client

    @api_request(
        log_message="Creating task_event_subscription {event_id}",
        out_schema=CreateTaskEventSubscriptionResponsePayload
    )
    def create_task_event_subscription(
        self,
        process_id: StrictStr,
        task_id: StrictStr,
        event_id: StrictStr,
        payload: Any = None,
        whatsapp_integration: StrictBool = False,
        email_integration: StrictBool = False,
        webhook_integration: StrictBool = False,
        whatsapp_recipients: Optional[Union[List[WhatsappRecipient], None]] = None,
        email_recipients: Optional[Union[List[StrictStr], None]] = None,
        webhook_url: Optional[Union[StrictStr, None]] = None,
        task_exception_id: Optional[StrictStr] = None
    ) -> CreateTaskEventSubscriptionResponsePayload:
        res: Any = requests.post(
            url=f"{self.api_client.ROOT_URL}{self.CREATE_TASK_EVENT_SUBSCRIPTION_URL.format(process_id, task_id)}",
            json=CreateTaskEventSubscriptionRequestPayload(
                event_id=event_id,
                whatsapp_integration=whatsapp_integration,
                email_integration=email_integration,
                webhook_integration=webhook_integration,
                payload=payload,
                whatsapp_recipients=whatsapp_recipients,
                email_recipients=email_recipients,
                webhook_url=webhook_url,
                task_exception_id=task_exception_id
            ).model_dump(),
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res

    @api_request(
        log_message="Deleting task_event_subscription {task_event_subscription_id}",
        out_schema=DeleteTaskEventSubscriptionResponsePayload
    )
    def delete_task_event_subscription(
        self,
        process_id: StrictStr,
        task_id: StrictStr,
        task_event_subscription_id: StrictStr
    ) -> DeleteTaskEventSubscriptionResponsePayload:
        res: Any = requests.delete(
            url=f"{self.api_client.ROOT_URL}{self.DELETE_TASK_EVENT_SUBSCRIPTION_URL.format(process_id, task_id, task_event_subscription_id)}",
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res

    @api_request(log_message="Getting task_event_subscriptions", out_schema=GetTaskEventSubscriptionsResponsePayload)
    def get_task_event_subscriptions(
        self,
        process_id: StrictStr,
        task_id: StrictStr
    ) -> GetTaskEventSubscriptionsResponsePayload:
        res: Any = requests.get(
            url=f"{self.api_client.ROOT_URL}{self.GET_TASK_EVENT_SUBSCRIPTIONS_URL.format(process_id, task_id)}",
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res

    @api_request(
        log_message="Updating task_event_subscription {task_event_subscription_id}",
        out_schema=UpdateTaskEventSubscriptionResponsePayload
    )
    def update_task_event_subscription(
        self,
        process_id: StrictStr,
        task_event_subscription_id: StrictStr,
        task_id: StrictStr,
        event_id: StrictStr,
        payload: Any = None,
        whatsapp_integration: StrictBool = False,
        email_integration: StrictBool = False,
        webhook_integration: StrictBool = False,
        whatsapp_recipients: Optional[Union[List[WhatsappRecipient], None]] = None,
        email_recipients: Optional[Union[List[StrictStr], None]] = None,
        webhook_url: Optional[Union[StrictStr, None]] = None,
        task_exception_id: Optional[StrictStr] = None
    ) -> UpdateTaskEventSubscriptionResponsePayload:
        res: Any = requests.put(
            url=f"{self.api_client.ROOT_URL}{self.UPDATE_TASK_EVENT_SUBSCRIPTION_URL.format(process_id, task_id, task_event_subscription_id)}",
            json=UpdateTaskEventSubscriptionRequestPayload(
                event_id=event_id,
                whatsapp_integration=whatsapp_integration,
                email_integration=email_integration,
                webhook_integration=webhook_integration,
                payload=payload,
                whatsapp_recipients=whatsapp_recipients,
                email_recipients=email_recipients,
                webhook_url=webhook_url,
                task_exception_id=task_exception_id
            ).model_dump(),
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res



import requests

from ergondata_executions.v1.auth.controller import AuthController
from ergondata_executions.v1.worker.interfaces import *
from ergondata_executions.v1.decorators import *


class WorkerController:

    CREATE_WORKER_URL = "processes/{0}/workers"
    GET_WORKERS_URL = "processes/{0}/workers"
    DELETE_WORKER_URL = "processes/{0}/workers/{1}"

    def __init__(self, api_client: AuthController):
        self.api_client = api_client

    @api_request(log_message="Creating worker", out_schema=CreateWorkerResponsePayload)
    def create_worker(self, process_id: StrictStr, friendly_name: StrictStr) -> CreateWorkerResponsePayload:
        res: Any = requests.post(
            url=f"{self.api_client.ROOT_URL}{self.CREATE_WORKER_URL.format(process_id)}",
            json={"first_name": friendly_name},
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res

    @api_request(log_message="Deleting worker", out_schema=DeleteWorkerResponsePayload)
    def delete_worker(self, process_id: StrictStr, worker_id: StrictStr) -> DeleteWorkerResponsePayload:
        res: Any = requests.delete(
            url=f"{self.api_client.ROOT_URL}{self.DELETE_WORKER_URL.format(process_id, worker_id)}",
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res

    @api_request(log_message="Getting workers", out_schema=GetWorkersResponsePayload)
    def get_workers(self, process_id: StrictStr) -> GetWorkersResponsePayload:
        res: Any = requests.get(
            url=f"{self.api_client.ROOT_URL}{self.GET_WORKERS_URL.format(process_id)}",
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res



import requests
from requests import Response

from ergondata_executions.v1.auth.controller import AuthController
from ergondata_executions.v1.process.interfaces import *
from ergondata_executions.v1.decorators import *

class ProcessController:

    CREATE_PROCESS_URL = "processes"
    GET_PROCESSES_URL = "processes"
    DELETE_PROCESS_URL = "processes/{0}"
    UPDATE_PROCESS_URL = "processes/{0}"

    def __init__(self, api_client: AuthController):
        self.api_client = api_client

    @api_request(log_message="Creating process {name}", out_schema=CreateProcessResponsePayload)
    def create_process(self, workspace_id: StrictStr, name: StrictStr, description: StrictStr) -> CreateProcessResponsePayload:
        res: Any = requests.post(
            url=f"{self.api_client.ROOT_URL}{self.CREATE_PROCESS_URL}",
            json=CreateProcessRequestPayload(
                name=name,
                description=description,
                workspace_id=workspace_id
            ).model_dump(),
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res

    @api_request(log_message="Deleting process {process_id}", out_schema=DeleteProcessResponsePayload)
    def delete_process(self, process_id: StrictStr) -> DeleteProcessResponsePayload:
        res: Any = requests.delete(
            url=f"{self.api_client.ROOT_URL}{self.DELETE_PROCESS_URL.format(process_id)}",
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res

    @api_request(log_message="Getting processes", out_schema=GetProcessResponsePayload)
    def get_processes(self) -> GetProcessResponsePayload:
        res: Any = requests.get(
            url=f"{self.api_client.ROOT_URL}{self.GET_PROCESSES_URL}",
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res

    @api_request(log_message="Updating process {name}", out_schema=UpdateProcessResponsePayload)
    def update_process(
        self,
        process_id: StrictStr,
        workspace_id: StrictStr,
        name: StrictStr,
        description: StrictStr
    ) -> UpdateProcessResponsePayload:
        res: Any = requests.put(
            url=f"{self.api_client.ROOT_URL}{self.UPDATE_PROCESS_URL.format(process_id)}",
            json=UpdateProcessRequestPayload(
                name=name,
                description=description,
                workspace_id=workspace_id
            ).model_dump(),
            headers=self.api_client.auth_header,
            timeout=self.api_client.timeout
        )
        return res



from datetime import datetime
from ergondata_executions.interfaces import *
from ergondata_executions.v1.workspace.interfaces import Workspace
from ergondata_executions.v1.process.interfaces import Process

class QueueItemStatus(BaseModel):
    id: Literal["success", "reset", "released", "business_exception", "system_error", "processing", "pending"]
    name: StrictStr


class Queue(BaseModel):
    id: StrictStr
    name: StrictStr
    description: StrictStr
    process: Process
    allow_to_include_repeated_queue_item: StrictBool
    allow_to_include_repeated_queue_item_success: StrictBool
    queue_item_max_retries_within_execution: StrictInt
    queue_item_max_retries_outside_execution: StrictInt
    created_at: datetime
    updated_at: datetime


class CreateQueueRequestPayload(BaseModel):
    name: StrictStr
    description: StrictStr
    allow_to_include_repeated_queue_item: Optional[StrictBool] = False
    allow_to_include_repeated_queue_item_success: Optional[StrictBool] = False
    queue_item_max_retries_within_execution: Optional[StrictInt] = 0
    queue_item_max_retries_outside_execution: Optional[StrictInt] = 0
    skip_ongoing_queue_item: Optional[StrictBool] = False

class UpdateQueueRequestPayload(CreateQueueRequestPayload):
    pass

class CreateQueueResponsePayload(APIBaseResponse):
    data: Optional[Queue] = None

class DeleteQueueResponsePayload(APIBaseResponse):
    queue_id: Optional[StrictStr] = None


class GetQueuesResponsePayload(APIBaseResponse):
    data: List[Queue]

class UpdateQueueResponsePayload(CreateQueueResponsePayload):
    data: Workspace


from ergondata_executions.interfaces import *
from ergondata_executions.v1.workspace.interfaces import Workspace

class Process(BaseModel):
    id: StrictStr
    name: StrictStr
    description: StrictStr
    workspace: Optional[Workspace] = None
    created_at: datetime
    updated_at: datetime


class CreateProcessRequestPayload(BaseModel):
    name: StrictStr
    description: StrictStr
    workspace_id: StrictStr

class UpdateProcessRequestPayload(CreateProcessRequestPayload):
    pass

class CreateProcessResponsePayload(APIBaseResponse):
    data: Optional[Process] = None

class DeleteProcessResponsePayload(APIBaseResponse):
    process_id: Optional[StrictStr] = None


class GetProcessResponsePayload(APIBaseResponse):
    data: List[Process]

class UpdateProcessResponsePayload(CreateProcessResponsePayload):
    data: Workspace